EXPORTS
	??1CSIPExSIPClientEstablishingState@@UAE@XZ @ 1 NONAME ; CSIPExSIPClientEstablishingState::~CSIPExSIPClientEstablishingState(void)
	??1CSIPExSIPClientOfferingState@@UAE@XZ @ 2 NONAME ; CSIPExSIPClientOfferingState::~CSIPExSIPClientOfferingState(void)
	??1CSIPExSIPEstablishedState@@UAE@XZ @ 3 NONAME ; CSIPExSIPEstablishedState::~CSIPExSIPEstablishedState(void)
	??1CSIPExSIPIdleState@@UAE@XZ @ 4 NONAME ; CSIPExSIPIdleState::~CSIPExSIPIdleState(void)
	??1CSIPExSIPServerEstablishingState@@UAE@XZ @ 5 NONAME ; CSIPExSIPServerEstablishingState::~CSIPExSIPServerEstablishingState(void)
	??1CSIPExSIPServerOfferingState@@UAE@XZ @ 6 NONAME ; CSIPExSIPServerOfferingState::~CSIPExSIPServerOfferingState(void)
	??1CSIPExSIPTerminatingState@@UAE@XZ @ 7 NONAME ; CSIPExSIPTerminatingState::~CSIPExSIPTerminatingState(void)
	?AcceptInviteL@CSIPExSIPEngine@@QAEXABVTInetAddr@@@Z @ 8 NONAME ; void CSIPExSIPEngine::AcceptInviteL(class TInetAddr const &)
	?CancelInviteL@CSIPExSIPEngine@@QAEXXZ @ 9 NONAME ; void CSIPExSIPEngine::CancelInviteL(void)
	?CreateIML@CSIPExSIPEngine@@QAEXABVTDesC8@@0@Z @ 10 NONAME ; void CSIPExSIPEngine::CreateIML(class TDesC8 const &, class TDesC8 const &)
	?DeclineInviteL@CSIPExSIPEngine@@QAEXXZ @ 11 NONAME ; void CSIPExSIPEngine::DeclineInviteL(void)
	?DisableProfileL@CSIPExSIPEngine@@QAEXXZ @ 12 NONAME ; void CSIPExSIPEngine::DisableProfileL(void)
	?EnableProfileL@CSIPExSIPEngine@@QAEHXZ @ 13 NONAME ; int CSIPExSIPEngine::EnableProfileL(void)
	?EndSessionL@CSIPExSIPEngine@@QAEXXZ @ 14 NONAME ; void CSIPExSIPEngine::EndSessionL(void)
	?NewL@CSIPExSIPClientEstablishingState@@SAPAV1@XZ @ 15 NONAME ; class CSIPExSIPClientEstablishingState * CSIPExSIPClientEstablishingState::NewL(void)
	?NewL@CSIPExSIPClientOfferingState@@SAPAV1@XZ @ 16 NONAME ; class CSIPExSIPClientOfferingState * CSIPExSIPClientOfferingState::NewL(void)
	?NewL@CSIPExSIPEngine@@SAPAV1@VTUid@@PAVMSIPExSIPEngineObserver@@@Z @ 17 NONAME ; class CSIPExSIPEngine * CSIPExSIPEngine::NewL(class TUid, class MSIPExSIPEngineObserver *)
	?NewL@CSIPExSIPEstablishedState@@SAPAV1@XZ @ 18 NONAME ; class CSIPExSIPEstablishedState * CSIPExSIPEstablishedState::NewL(void)
	?NewL@CSIPExSIPIdleState@@SAPAV1@XZ @ 19 NONAME ; class CSIPExSIPIdleState * CSIPExSIPIdleState::NewL(void)
	?NewL@CSIPExSIPServerEstablishingState@@SAPAV1@XZ @ 20 NONAME ; class CSIPExSIPServerEstablishingState * CSIPExSIPServerEstablishingState::NewL(void)
	?NewL@CSIPExSIPServerOfferingState@@SAPAV1@XZ @ 21 NONAME ; class CSIPExSIPServerOfferingState * CSIPExSIPServerOfferingState::NewL(void)
	?NewL@CSIPExSIPTerminatingState@@SAPAV1@XZ @ 22 NONAME ; class CSIPExSIPTerminatingState * CSIPExSIPTerminatingState::NewL(void)
	?SendInviteL@CSIPExSIPEngine@@QAEXABVTDesC8@@@Z @ 23 NONAME ; void CSIPExSIPEngine::SendInviteL(class TDesC8 const &)

